﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System.Threading;

namespace TamiFlu.Core
{
    public static class InterlockedUtility
    {
        #region Methods

        public static void Max(ref float storage, float value)
        {
            var currentValue = storage;
            while (true)
            {
                if (currentValue >= value)
                {
                    break;
                }

                var oldValue = Interlocked.CompareExchange(ref storage, value, currentValue);
                // ReSharper disable once CompareOfFloatsByEqualityOperator
                if (oldValue == currentValue)
                {
                    break;
                }

                currentValue = oldValue;
            }
        }

        public static void Min(ref float storage, float value)
        {
            var currentValue = storage;
            while (true)
            {
                if (currentValue <= value)
                {
                    break;
                }

                var oldValue = Interlocked.CompareExchange(ref storage, value, currentValue);
                // ReSharper disable once CompareOfFloatsByEqualityOperator
                if (oldValue == currentValue)
                {
                    break;
                }

                currentValue = oldValue;
            }
        }

        public static void Add(ref float storage, float value)
        {
            var currentValue = storage;
            while (true)
            {
                var oldValue = Interlocked.CompareExchange(ref storage, currentValue + value, currentValue);
                // ReSharper disable once CompareOfFloatsByEqualityOperator
                if (oldValue == currentValue)
                {
                    break;
                }

                currentValue = oldValue;
            }
        }

        #endregion
    }
}
