﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System.ComponentModel;
using JetBrains.Annotations;
using TamiFlu.Core.UI;
using TamiFlu.Interfaces;
using TamiFlu.Interfaces.Simulation;
using TamiFlu.Interfaces.Simulation.Forces;

namespace TamiFlu.Core.Simulation.Forces
{
    [TypeConverter(typeof (ExpandableObjectConverter)), Plugin("Gravity", "1.0", "Stefan Band"), UsedImplicitly]
    internal sealed class GravityForce : IExternalForce
    {
        #region Properties

        [Browsable(true), TypeConverter(typeof(VectorTypeConverter)), UsedImplicitly]
        public Vector Gravity { get; set; } = new Vector(0f, -9.81f);

        #endregion

        #region Implementation of IForce

        /// <inheritdoc />
        public void ApplyToFluidParticles(IParticleContext particleContext)
        {
            var gravityAcceleration = Gravity;
            particleContext.ForEachFluidParticleInParallel((ref FluidParticle f) => f.Acceleration += gravityAcceleration);
        }

        #endregion
    }
}
