﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using JetBrains.Annotations;
using TamiFlu.Interfaces;
using TamiFlu.Interfaces.Simulation;
using TamiFlu.Interfaces.Simulation.Forces;

namespace TamiFlu.Core.Simulation.Forces
{
    [Plugin("Laminar Viscosity", "1.0", "Stefan Band"), UsedImplicitly]
    internal sealed class ViscousForce : IViscousForce
    {
        #region Implementation of IForce

        /// <inheritdoc />
        public void ApplyToFluidParticles(IParticleContext particleContext)
        {
            var kernel = particleContext.Kernel;
            var epsilon = 0.01f * GlobalConstants.Instance.RestVolume;
            var fluidParticles = particleContext.FluidParticles;
            var boundaryParticles = particleContext.BoundaryParticles;

            particleContext.ForEachFluidParticleInParallel((ref FluidParticle f) =>
            {
                var velocityLaplace = Vector.Zero;

                foreach (var ffIndex in f.FluidNeighbors)
                {
                    ref readonly var ff = ref fluidParticles[ffIndex];
                    var fffPosition = f.Position - ff.Position;
                    velocityLaplace += (ff.Properties.Viscosity * ff.Volume * Vector.Dot(in fffPosition, kernel.ComputeGradient(f.Position, ff.Position)) / (Vector.LengthSquared(in fffPosition) + epsilon)) * (f.Velocity - ff.Velocity);
                }

                // Akinci boundary handling
                foreach (var fbIndex in f.BoundaryNeighbors)
                {
                    ref readonly var fb = ref boundaryParticles[fbIndex];
                    var ffbPosition = f.Position - fb.Position;
                    velocityLaplace += (fb.Properties.Viscosity * fb.Volume * Vector.Dot(in ffbPosition, kernel.ComputeGradient(f.Position, fb.Position)) / (Vector.LengthSquared(in ffbPosition) + epsilon)) * (f.Velocity - fb.Velocity);
                }

                f.Acceleration += 2f * velocityLaplace;
            });
        }

        #endregion
    }
}
