﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using JetBrains.Annotations;
using TamiFlu.Interfaces.Simulation;
using TamiFlu.Interfaces.Simulation.Kernel;

namespace TamiFlu.Core.Simulation.Kernel
{
    public static class KernelExtensions
    {
        #region Methods

        [NotNull]
        public static IEnumerable<Vector> SampleVolume([NotNull] IKernel kernel, out Vector centerPosition, out float sampleVolume)
        {
            return SampleVolume(kernel.SupportFactor, kernel.SupportLength, out centerPosition, out sampleVolume);
        }

        [NotNull]
        public static IEnumerable<Vector> SampleVolume(float factor, float size, out Vector centerPosition, out float sampleVolume)
        {
            var positions = new List<Vector>();
            var spacing = size / (float) Math.Truncate(factor);

            sampleVolume = spacing * spacing;
            for (var y = -size; y <= size; y += spacing)
            {
                for (var x = -size; x <= size; x += spacing)
                {
                    positions.Add(new Vector(x, y));
                }
            }

            // search for closest position to origin
            centerPosition = positions.First();
            foreach (var position in positions)
            {
                if (Vector.LengthSquared(in position) < Vector.LengthSquared(in centerPosition))
                {
                    centerPosition = position;
                }
            }

            return positions;
        }

        public static float ComputeNormalizationFactor<T>([NotNull] T kernel) where T : IKernel
        {
            var sum = 0.0f;
            foreach (var x2 in SampleVolume(kernel, out var x1, out var sampleVolume))
            {
                sum += kernel.ComputeValue(x1, x2);
            }
            return 1f / sum;
        }

        #endregion
    }
}
