#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using TamiFlu.Interfaces.Simulation;

namespace TamiFlu.Core.Simulation.Neighborhood
{
    internal struct BoundingBox
    {
        #region Fields

        public static readonly BoundingBox Invalid = new BoundingBox(new Vector(float.MaxValue), new Vector(float.MinValue));

        /// <summary>
        /// The minimum position of the bounding box.
        /// </summary>
        public readonly Vector Minimum;

        /// <summary>
        /// The maximum position of the bounding box
        /// </summary>
        public readonly Vector Maximum;

        #endregion

        #region Properties

        /// <summary>
        /// Gets the size of this bounding box.
        /// </summary>
        public Vector Size => Maximum - Minimum;

        #endregion

        #region Constructor

        public BoundingBox(Vector minimum, Vector maximum)
        {
            Minimum = minimum;
            Maximum = maximum;
        }

        #endregion

        #region Methods

        public bool Contains(Vector position)
        {
            for (var dimension = 0; dimension < Vector.Dimensions; ++dimension)
            {
                if (position[dimension] < Minimum[dimension] || position[dimension] > Maximum[dimension])
                {
                    return false;
                }
            }

            return true;
        }

        public static BoundingBox Combine(BoundingBox lhs, BoundingBox rhs)
        {
            return new BoundingBox(Vector.Min(lhs.Minimum, rhs.Minimum), Vector.Max(lhs.Maximum, rhs.Maximum));
        }

        #endregion
    }
}