﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Text;
using JetBrains.Annotations;
using TamiFlu.Interfaces.Neighborhood;
using TamiFlu.Interfaces.Simulation;

namespace TamiFlu.Core.Simulation.Neighborhood
{
    internal sealed class NeighborSet : List<ParticleIndex>, INeighborSet
    {
        #region Methods

        [NotNull]
        public byte[] Compress()
        {
            if (Count != 0)
            {
                //Sort(Comparer<ParticleIndex>.Default);

                using (var memoryStream = new MemoryStream())
                {
                    using (var binaryWriter = new BinaryWriter(memoryStream, new UTF8Encoding(false, true), true))
                    {
                        var offset = this[0].Value;
                        foreach (var index in this)
                        {
                            binaryWriter.Write(index.Value - offset);
                            offset = index.Value;
                        }
                    }
                    memoryStream.Position = 0;


                    using (var compressedMemoryStream = new MemoryStream())
                    {
                        using (var gZipStream = new GZipStream(compressedMemoryStream, CompressionLevel.NoCompression, true))
                        {
                            gZipStream.Write(memoryStream.GetBuffer(), 0, (int) memoryStream.Length);
                            return compressedMemoryStream.ToArray();
                        }
                    }
                }
            }

            return Array.Empty<byte>();
        }

        #endregion
    }
}
