﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System;
using TamiFlu.Interfaces;
using TamiFlu.Interfaces.Simulation;

namespace TamiFlu.Core.Simulation
{
    [Plugin("Serializer", "1.0", "Stefan Band")]
    internal sealed class ParticleSerializer : IParticleSerializer
    {
        #region Implementation of IParticleSerializer

        /// <inheritdoc />
        public void Load(IParticleContext particleContext, string fileName)
        {
            particleContext.Clear();

            // TODO load from scene file

            var globalConstants = GlobalConstants.Instance;
            globalConstants.Spacing = 0.0125f;
            
            var particleSource = new ParticleSource();

            // create fluid particles
            var fluidProperties = new FluidParticlePhysicsProperties
                                  {
                                      RestDensity = 1000f,
                                      SurfaceTension = 0.1f,
                                      Viscosity = 1e-3f
                                  };
            particleSource.SampleFluidParticleBlock(particleContext, new Vector(0f, -0.48f), new Vector(1f), fluidProperties);

            // create boundary particles
            var boundaryProperties = new BoundaryParticlePhysicsProperties
                                     {
                                         Viscosity = 1e-3f
                                     };
            particleSource.SampleBoundaryParticleBlock(particleContext, new Vector(0f, -1f), new Vector(1.5f, globalConstants.Spacing), boundaryProperties);
            particleSource.SampleBoundaryParticleBlock(particleContext, new Vector(-0.75f, -0.5f), new Vector(globalConstants.Spacing, 1f), boundaryProperties);
            particleSource.SampleBoundaryParticleBlock(particleContext, new Vector(0.75f, -0.5f), new Vector(globalConstants.Spacing, 1f), boundaryProperties);
        }

        /// <inheritdoc />
        public void Save(IParticleContext particleContext, string fileName)
        {
            // TODO save to scene file
            throw new NotImplementedException();
        }

        #endregion
    }
}
