﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System;
using System.ComponentModel;
using TamiFlu.Interfaces;
using TamiFlu.Interfaces.Simulation.Solver;

namespace TamiFlu.Core.Simulation.Solver
{
    [TypeConverter(typeof (ExpandableObjectConverter)), Plugin("State Equation", "1.0", "Stefan Band")]
    internal sealed class StateEquation : IStateEquation
    {
        #region Constants

        private const float DefaultK1 = 6400f;
        private const float DefaultK2 = 7f;

        #endregion

        #region Properties

        [Browsable(true), DefaultValue(DefaultK1)]
        public float K1 { get; set; } = DefaultK1;

        [Browsable(true), DefaultValue(DefaultK2)]
        public float K2 { get; set; } = DefaultK2;

        #endregion

        #region Implementation of IStateEquation

        /// <inheritdoc />
        public float ComputePressure(float density, float restDensity)
        {
            return Math.Max(0f, K1 * ((float) Math.Pow(density / restDensity, K2) - 1f));
        }

        #endregion

        #region Methods

        /// <inheritdoc />
        public override string ToString()
        {
            return "k\u2081(ρ/ρ\u2080 - 1)ᴷ²";
        }

        #endregion
    }
}