﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using JetBrains.Annotations;
using TamiFlu.Interfaces;

namespace TamiFlu.Core.Simulation.SystemSolver
{
    #region Delegates

    public delegate void ComputeDiagonalElements([NotNull] ref float[] diagonalElements);
    public delegate void ComputeSourceTerms([NotNull] ref float[] sourceTerms);
    public delegate void ComputeMatrixVectorProduct([NotNull] float[] vectors, [NotNull] ref float[] matrixVectorProducts);
    public delegate void InitializeVector([NotNull] ref float[] vectors);

    #endregion

    internal interface ISystemSolverResult
    {
        #region Properties

        int NumberOfIterations { get; }
        
        bool IsConverged { get; }

        float SolutionError { get; }

        [NotNull]
        float[] SolutionVector { get; }

        #endregion
    }

    internal interface ISystemSolver
    {
        #region Properties

        ComputeDiagonalElements ComputeDiagonalElementFunc { get; set; }

        ComputeSourceTerms ComputeSourceTermFunc { get; set; }

        InitializeVector InitializeVectorFunc { get; set; }

        ComputeMatrixVectorProduct ComputeMatrixVectorProductFunc { get; set; }

        #endregion

        #region Methods

        /// <summary>
        /// Solve the system described by the functors.
        /// </summary>
        /// <param name="parallelContext"></param>
        /// <returns>the solution vector</returns>
        [NotNull]
        ISystemSolverResult SolveSystem([NotNull] IParallelContext parallelContext);

        #endregion
    }
}
