﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion

using TamiFlu.Interfaces.Simulation;

namespace TamiFlu.Core.UI.Rendering
{
    internal sealed class Camera
    {
        #region Properties

        public System.Numerics.Vector2 Offset { get; set; }

        public Eto.Drawing.Rectangle Rectangle { get; set; }

        public float Zoom { get; set; } = 1.0f;

        public int Width => Rectangle.Width;

        public int Height => Rectangle.Height;

        public System.Numerics.Vector2 Size => new System.Numerics.Vector2(Width, Height);

        public System.Numerics.Vector2 Location => new System.Numerics.Vector2(Rectangle.Left, Rectangle.Top) + Offset;

        public System.Numerics.Matrix4x4 ProjectionMatrix => System.Numerics.Matrix4x4.CreateOrthographicOffCenter(-Zoom, Zoom, Zoom, -Zoom, 0.0f, 1.0f);

        #endregion

        #region Methods

        public Vector Unproject(System.Numerics.Vector2 windowCoordinates)
        {
            if (System.Numerics.Matrix4x4.Invert(ProjectionMatrix, out var inverseP))
            {
                var ndc = ToNormalizedDeviceCoordinates(in windowCoordinates, Location, Size);
                return Vector.Transform(in ndc, in inverseP);
            }

            return Vector.Zero;
        }

        public static Vector ToNormalizedDeviceCoordinates(in System.Numerics.Vector2 windowCoordinates, in System.Numerics.Vector2 location, in System.Numerics.Vector2 size)
        {
            var ndc = (windowCoordinates - location) / (size * 0.5f) - System.Numerics.Vector2.One;
            return new Vector(ndc.X, ndc.Y);
        }

        public System.Numerics.Vector2 ToWindowCoordinates(in Vector normalizedDeviceCoordinate)
        {
            return ToWindowCoordinates(in normalizedDeviceCoordinate, Location, Size);
        }

        public static System.Numerics.Vector2 ToWindowCoordinates(in Vector normalizedDeviceCoordinate, in System.Numerics.Vector2 location, in System.Numerics.Vector2 size)
        {
            var ndc = new System.Numerics.Vector2(normalizedDeviceCoordinate.X, normalizedDeviceCoordinate.Y);
            return location + (size * 0.5f) * (ndc + System.Numerics.Vector2.One);
        }

        #endregion
    }
}
