﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System.ComponentModel;
using JetBrains.Annotations;
using TamiFlu.Interfaces;
using TamiFlu.Interfaces.Rendering;
using TamiFlu.Interfaces.Simulation;

namespace TamiFlu.Core.UI.Rendering.Color
{
    [TypeConverter(typeof(ExpandableObjectConverter)), Plugin("Velocity Color", "1.0", "Stefan Band"), UsedImplicitly]
    internal sealed class VelocityColorComputer : BaseColorTableComputer
    {
        #region Overrides of BaseColorTableComputer

        /// <inheritdoc />
        protected override string AttributeName => "Velocity";

        /// <inheritdoc />
        protected override ARGBColorTable FluidColorTable => ARGBColorTable.Jet;

        /// <inheritdoc />
        protected override ARGBColorTable BoundaryColorTable => ARGBColorTable.Jet;

        /// <inheritdoc />
        protected override float GetFluidValue(ref FluidParticle f)
        {
            return Vector.Length(f.Velocity);
        }

        /// <inheritdoc />
        protected override float GetBoundaryValue(ref BoundaryParticle b)
        {
            return Vector.Length(b.Velocity);
        }

        #endregion
    }
}
