﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System;
using System.ComponentModel;
using System.Globalization;
using System.Text;
using TamiFlu.Interfaces.Simulation;

namespace TamiFlu.Core.UI
{
    internal sealed class VectorTypeConverter : ExpandableObjectConverter
    {
        #region Methods

        /// <inheritdoc />
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            if (sourceType == typeof(string))
            {
                return true;
            }

            return base.CanConvertFrom(context, sourceType);
        }

        /// <inheritdoc />
        public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture, object value)
        {
            if (value is string valueAsString)
            {
                var values = valueAsString.Split(new[] {", "}, StringSplitOptions.RemoveEmptyEntries);

                switch (values.Length)
                {
                    case 1:
                        return new Vector(float.Parse(values[0], culture));

                    case 2:
                        return new Vector(float.Parse(values[0], culture), float.Parse(values[1], culture));

                    default:
                        throw new ArgumentOutOfRangeException(nameof(values));
                }
            }

            return base.ConvertFrom(context, culture, value);
        }

        /// <inheritdoc />
        public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destinationType)
        {
            if (destinationType == typeof(string))
            {
                var stringBuilder = new StringBuilder();
                if (Vector.Dimensions >= 1)
                {
                    var vector = (Vector) value;
                    stringBuilder.Append(vector[0].ToString(culture));
                    for (var i = 1; i < Vector.Dimensions; ++i)
                    {
                        stringBuilder.Append(", ");
                        stringBuilder.Append(vector[i].ToString(culture));
                    }
                }
                return stringBuilder.ToString();
            }

            return base.ConvertTo(context, culture, value, destinationType);
        }

        #endregion
    }
}
