﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System;
using System.Collections.Generic;
using JetBrains.Annotations;

namespace TamiFlu.Interfaces
{
    public interface IParallelContext
    {
        #region Properties

        int NumThreads { get; set; }

        #endregion

        #region Methods

        void ForEachRange(int startIndex, int endIndex, [NotNull] Action<int, int> action);

        void ForEach<T, TList>([NotNull] TList source, [NotNull] Action<T> action) where TList : class, IReadOnlyList<T>;
        
        void ForEach<T>([NotNull] IEnumerable<T> source, [NotNull] Action<T> action);

        void ForEach<T>([NotNull] IEnumerable<T> source, int minChunkSize, int maxChunkSize, [NotNull] Action<T> action);

        TResult ReduceRange<TResult>(int startIndex, int endIndex, [NotNull] Func<int, TResult> action, [NotNull] Func<TResult, TResult, TResult> reduction, TResult initValue) where TResult : struct;

        TResult Reduce<TResult, T, TList>([NotNull] TList source, [NotNull] Func<T, TResult> action, [NotNull] Func<TResult, TResult, TResult> reduction, TResult initValue) where TList : class, IReadOnlyList<T> where TResult : struct;

        [NotNull]
        TResult CombineRange<TResult>(int startIndex, int endIndex, [NotNull] Action<int, TResult> action, [NotNull] Func<TResult, TResult, TResult> reduction, [NotNull] Func<TResult> createInitValue) where TResult : class;

        [NotNull]
        TResult Combine<TResult, T, TList>([NotNull] TList source, [NotNull] Action<T, TResult> action, [NotNull] Func<TResult, TResult, TResult> reduction, [NotNull] Func<TResult> createInitValue) where TList : class, IReadOnlyList<T> where TResult : class;

        #endregion
    }
}
