﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using JetBrains.Annotations;
using TamiFlu.Interfaces.Simulation;

namespace TamiFlu.Interfaces.Neighborhood
{
    public interface IParticleNeighborhood
    {
        #region Properties

        /// <summary>
        /// Gets whether SearchNeighbors has been called.
        /// </summary>
        bool HasNeighbors { get; }

        #endregion

        #region Methods

        /// <summary>
        /// Search and store all particle neighbor sets.
        /// </summary>
        /// <param name="particleContext"></param>
        void SearchNeighbors([NotNull] IParticleContext particleContext);

        /// <summary>
        /// Search the fluid and boundary particles that are neighbors
        /// of <paramref name="position"/> and execute <paramref name="fluidParticleAction"/>
        /// or <paramref name="boundaryParticleAction"/> for each found neighbor.
        /// </summary>
        void SearchNeighbors([NotNull] IParticleContext particleContext, Vector position, [NotNull] FluidParticleAction fluidParticleAction, [NotNull] BoundaryParticleAction boundaryParticleAction);

        #endregion
    }
}
