﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System;

namespace TamiFlu.Interfaces.Rendering
{
    public struct ARGBColor : IEquatable<ARGBColor>
    {
        #region Static Fields

        public static readonly ARGBColor Empty = FromARGB(0, 0, 0, 0);
        public static readonly ARGBColor Black = FromRGB(0, 0, 0);
        public static readonly ARGBColor White = FromRGB(0xFF, 0xFF, 0xFF);
        public static readonly ARGBColor Red = FromRGB(0xFF, 0, 0);
        public static readonly ARGBColor Green = FromRGB(0, 0xFF, 0);
        public static readonly ARGBColor Blue = FromRGB(0, 0, 0xFF);
        public static readonly ARGBColor Cyan = FromRGB(0, 0xFF, 0xFF);
        public static readonly ARGBColor Darkblue = FromRGB(0, 0, 0x8B);
        public static readonly ARGBColor Darkred = FromRGB(0x8B, 0, 0);
        public static readonly ARGBColor Yellow = FromRGB(0xFF, 0xFF, 0);
        public static readonly ARGBColor Orange = FromRGB(0xFF, 0xA5, 0);
        public static readonly ARGBColor Magenta = FromRGB(0xFF, 0, 0xFF);

        #endregion

        #region Fields

        public readonly byte A;
        public readonly byte R;
        public readonly byte G;
        public readonly byte B;

        #endregion
       
        #region Constructor

        private ARGBColor(byte a, byte r, byte g, byte b)
        {
            A = a;
            R = r;
            G = g;
            B = b;
        }

        #endregion

        #region Methods

        public static ARGBColor Interpolate(ref ARGBColor colorBegin, ref ARGBColor colorEnd, double t)
        {
            return FromARGB((byte) ((1.0 - t) * colorBegin.A + t * colorEnd.A),
                            (byte) ((1.0 - t) * colorBegin.R + t * colorEnd.R),
                            (byte) ((1.0 - t) * colorBegin.G + t * colorEnd.G),
                            (byte) ((1.0 - t) * colorBegin.B + t * colorEnd.B));
        }

        public static ARGBColor FromRGB(byte r, byte g, byte b)
        {
            return FromARGB(byte.MaxValue, r, g, b);
        }

        public static ARGBColor FromARGB(byte a, byte r, byte g, byte b)
        {
            return new ARGBColor(a, r, g, b);
        }

        /// <inheritdoc />
        public override string ToString()
        {
            return $"A = {A}, R = {R}, G = {G}, B = {B}";
        }

        /// <inheritdoc />
        public bool Equals(ARGBColor other)
        {
            return A == other.A && R == other.R && G == other.G && B == other.B;
        }

        /// <inheritdoc />
        public override bool Equals(object obj)
        {
            return !(obj is null) && (obj is ARGBColor && Equals((ARGBColor) obj));
        }

        /// <inheritdoc />
        public override int GetHashCode()
        {
            unchecked
            {
                var hashCode = A.GetHashCode();
                hashCode = (hashCode * 397) ^ R.GetHashCode();
                hashCode = (hashCode * 397) ^ G.GetHashCode();
                hashCode = (hashCode * 397) ^ B.GetHashCode();
                return hashCode;
            }
        }

        #endregion
    }
}
