﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System;
using JetBrains.Annotations;

namespace TamiFlu.Interfaces.Rendering
{
    public sealed class ARGBColorTable
    {
        #region Static Fields

        [NotNull] public static readonly ARGBColorTable Jet = new ARGBColorTable(new []
                                                                                 {
                                                                                     ARGBColor.Darkblue,
                                                                                     ARGBColor.Cyan,
                                                                                     ARGBColor.Yellow,
                                                                                     ARGBColor.Orange,
                                                                                     ARGBColor.Darkred
                                                                                 });

        [NotNull] public static readonly ARGBColorTable Hue = new ARGBColorTable(new []
                                                                                 {
                                                                                     ARGBColor.Red,
                                                                                     ARGBColor.Yellow,
                                                                                     ARGBColor.Green,
                                                                                     ARGBColor.Cyan,
                                                                                     ARGBColor.Blue,
                                                                                     ARGBColor.Magenta
                                                                                 });

        [NotNull] public static readonly ARGBColorTable Hot = new ARGBColorTable(new []
                                                                                 {
                                                                                     ARGBColor.Black,
                                                                                     ARGBColor.FromRGB(0x7F, 0, 0),
                                                                                     ARGBColor.FromRGB(0xFF, 0x7F, 0),
                                                                                     ARGBColor.FromRGB(0xFF, 0xFF, 0x7F),
                                                                                     ARGBColor.White
                                                                                 });

        [NotNull] public static readonly ARGBColorTable Gray = new ARGBColorTable(new []
                                                                                  {
                                                                                      ARGBColor.Black,
                                                                                      ARGBColor.White
                                                                                  });

        #endregion

        #region Properties

        [NotNull]
        public ARGBColor[] Colors { get; }

        #endregion

        #region Constructor

        public ARGBColorTable([NotNull] ARGBColor[] colors)
        {
            Colors = colors ?? throw new ArgumentNullException(nameof(colors));
        }

        #endregion

        #region Methods

        public ARGBColor InterpolateColor(double value, double minimum, double maximum)
        {
            var fractionIndex = Math.Min(Math.Max(value - minimum, 0.0) / (maximum - minimum), 1.0);
            var doubleIndex = double.IsNaN(fractionIndex) ? 0.0 : fractionIndex * (Colors.Length - 1);
            var integerIndex = Math.Truncate(doubleIndex);
            var beginIndex = (int) integerIndex;
            var endIndex = Math.Min(beginIndex + 1, Colors.Length - 1);
            return ARGBColor.Interpolate(ref Colors[beginIndex], ref Colors[endIndex], doubleIndex - integerIndex);
        }

        #endregion
    }
}
