﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using JetBrains.Annotations;

namespace TamiFlu.Interfaces.Simulation
{
    public sealed class GlobalConstants : INotifyPropertyChanged
    {
        #region Static Properties

        public static GlobalConstants Instance { get; } = new GlobalConstants();

        #endregion

        #region Fields

        private float m_Spacing;

        #endregion

        #region Properties

        public float Spacing
        {
            get => m_Spacing;
            set => SetProperty(ref m_Spacing, value);
        }

        public float RestVolume
        {
            get
            {
                #pragma warning disable 162
                // ReSharper disable once HeuristicUnreachableCode
                switch (Vector.Dimensions)
                {
                    case 2:
                        return Spacing * Spacing;

                    case 3:
                        return Spacing * Spacing * Spacing;

                    default:
                        throw new ArgumentOutOfRangeException();
                }
                #pragma warning restore 162
            }
        }

        #endregion

        #region Methods

        private void SetProperty<T>([CanBeNull] ref T field, [CanBeNull] T newValue, [CallerMemberName] string propertyName = null)
        {
            if (!EqualityComparer<T>.Default.Equals(field, newValue))
            {
                field = newValue;

                OnPropertyChanged(propertyName);
            }
        }

        #endregion

        #region Implementation of INotifyPropertyChanged

        /// <inheritdoc />
        public event PropertyChangedEventHandler PropertyChanged;

        [NotifyPropertyChangedInvocator]
        private void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            var propertyChanged = PropertyChanged;
            if (!(propertyChanged is null))
            {
                propertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        #endregion
    }
}
