﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System;
using JetBrains.Annotations;
using TamiFlu.Interfaces.Neighborhood;
using TamiFlu.Interfaces.Rendering;
using TamiFlu.Interfaces.Simulation.Kernel;
using TamiFlu.Interfaces.Simulation.Solver;

namespace TamiFlu.Interfaces.Simulation
{
    public interface IParticleContext
    {
        #region Properties

        [NotNull] IParallelContext Parallel { get; }

        [NotNull] ITimeStep TimeStep { get; }

        [NotNull] IFluidSolver FluidSolver { get; }

        [NotNull] IKernel Kernel { get; }

        [NotNull] IParticleNeighborhood Neighborhood { get; }

        [NotNull] IParticleSerializer Serializer { get; }

        [NotNull] IParticleColorComputer ColorComputer { get; }

        [NotNull] IPerformanceMeasurement PerformanceMeasurement { get; }

        [NotNull] FluidParticle[] FluidParticles { get; }

        [NotNull] BoundaryParticle[] BoundaryParticles { get; }

        [CanBeNull]
        ParticleIndex? PickedFluidParticle { get; set; }

        [CanBeNull]
        ParticleIndex? PickedBoundaryParticle { get; set; }

        #endregion

        #region Methods

        [CanBeNull]
        string GetDebugText();

        void Clear();

        ParticleIndex CreateFluidParticles([NotNull] FluidParticlePhysicsProperties properties, int count);

        ParticleIndex CreateBoundaryParticles([NotNull] BoundaryParticlePhysicsProperties properties, int count);

        void ReorderFluidParticles([NotNull] Func<ParticleIndex, ParticleIndex> action, bool ignoreAttributes);

        void ReorderBoundaryParticles([NotNull] Func<ParticleIndex, ParticleIndex> action, bool ignoreAttributes);

        void ForEachFluidParticle([NotNull] FluidParticleAction action);
        void ForEachFluidParticle([NotNull] FluidParticleIndexAction action);

        void ForEachBoundaryParticle([NotNull] BoundaryParticleAction action);
        void ForEachBoundaryParticle([NotNull] BoundaryParticleIndexAction action);

        void ForEachFluidParticleInParallel([NotNull] FluidParticleAction action);
        void ForEachFluidParticleInParallel([NotNull] FluidParticleIndexAction action);

        void ForEachBoundaryParticleInParallel([NotNull] BoundaryParticleAction action);
        void ForEachBoundaryParticleInParallel([NotNull] BoundaryParticleIndexAction action);


        T ForEachFluidParticleReduceInParallel<T>([NotNull] FluidParticleFunction<T> action, [NotNull] Func<T, T, T> reduction, T initValue) where T : struct;
        T ForEachFluidParticleReduceInParallel<T>([NotNull] FluidParticleIndexFunction<T> action, [NotNull] Func<T, T, T> reduction, T initValue) where T : struct;

        T ForEachBoundaryParticleReduceInParallel<T>([NotNull] BoundaryParticleFunction<T> action, [NotNull] Func<T, T, T> reduction, T initValue) where T : struct;
        T ForEachBoundaryParticleReduceInParallel<T>([NotNull] BoundaryParticleIndexFunction<T> action, [NotNull] Func<T, T, T> reduction, T initValue) where T : struct;
        
        #endregion
    }
}
