﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System;
using System.Collections.Generic;
using System.Diagnostics;
using JetBrains.Annotations;

namespace TamiFlu.Interfaces.Simulation
{
    public interface IPerformanceMeasurement : IEnumerable<KeyValuePair<string, Stopwatch>>
    {
        #region Properties

        int Count { get; }

        [NotNull]
        IEnumerable<string> Names { get; }

        [NotNull]
        IEnumerable<Stopwatch> Stopwatches { get; }

        [CanBeNull]
        Stopwatch Current { get; }

        #endregion

        #region Methods

        void Push();

        void Pop();

        TimeSpan Elapsed([NotNull] string name);

        void Reset();

        [NotNull]
        Stopwatch Reset([NotNull] string name);

        [NotNull]
        Stopwatch Restart([NotNull] string name);

        [NotNull]
        Stopwatch Start([NotNull] string name);

        void Measure([NotNull] string name, [NotNull] Action action);

        T Measure<T>([NotNull] string name, [NotNull] Func<T> action);

        #endregion
    }
}
