﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System;

namespace TamiFlu.Interfaces.Simulation
{
    public struct ParticleIndex : IEquatable<ParticleIndex>, IComparable<ParticleIndex>
    {
        #region Fields

        public readonly int Value;

        public static readonly ParticleIndex Zero = new ParticleIndex(0);
        public static readonly ParticleIndex One = new ParticleIndex(1);

        #endregion

        #region Constructor

        public ParticleIndex(int value)
        {
            Value = value;
        }

        #endregion

        #region Methods

        public static implicit operator int(ParticleIndex particleIndex)
        {
            return particleIndex.Value;
        }

        public static ParticleIndex operator++(ParticleIndex value)
        {
            return new ParticleIndex(value.Value + 1);
        }

        public static ParticleIndex operator +(ParticleIndex lhs, ParticleIndex rhs)
        {
            return new ParticleIndex(lhs.Value + rhs.Value);
        }

        public static ParticleIndex operator -(ParticleIndex lhs, ParticleIndex rhs)
        {
            return new ParticleIndex(lhs.Value - rhs.Value);
        }

        /// <inheritdoc />
        public override string ToString()
        {
            return Value.ToString();
        }

        /// <inheritdoc />
        public bool Equals(ParticleIndex other)
        {
            return Value == other.Value;
        }

        /// <inheritdoc />
        public override bool Equals(object obj)
        {
            return !(obj is null)
                   && obj is ParticleIndex
                   && Equals((ParticleIndex) obj);
        }

        /// <inheritdoc />
        public override int GetHashCode()
        {
            return Value.GetHashCode();
        }

        #endregion

        #region Implementation of IComparable<ParticleIndex>

        /// <inheritdoc />
        public int CompareTo(ParticleIndex other)
        {
            return Value.CompareTo(other.Value);
        }

        #endregion
    }
}
