﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using JetBrains.Annotations;

namespace TamiFlu.Interfaces.Simulation.Solver
{
    #region Delegates

    public delegate void FluidGradientEndAction(ref FluidParticle f, in ParticleIndex fIndex, in Vector pressureGradient);
    public delegate void FluidDivergenceEndAction(ref FluidParticle f, in ParticleIndex fIndex, float divergence);

    public delegate void BoundaryGradientEndAction(ref BoundaryParticle b, in ParticleIndex bIndex, in Vector pressureGradient);
    public delegate void BoundaryDivergenceEndAction(ref BoundaryParticle b, in ParticleIndex bIndex, float divergence);

    public delegate Vector GetFluidVector(in FluidParticle f, in ParticleIndex fIndex);
    public delegate float GetFluidScalar(in FluidParticle f, in ParticleIndex fIndex);
    public delegate Vector GetBoundaryVector(in BoundaryParticle f, in ParticleIndex bIndex);
    public delegate float GetBoundaryScalar(in BoundaryParticle f, in ParticleIndex bIndex);

    #endregion

    public interface IBoundaryHandling
    {
        #region Methods

        void ComputeVolume([NotNull] IParticleContext particleContext);

        void ComputeGradient([NotNull] IParticleContext particleContext, [NotNull] GetFluidScalar getFluidScalar, [NotNull] GetBoundaryScalar getBoundaryScalar, [CanBeNull] FluidGradientEndAction fluidAction, [CanBeNull] BoundaryGradientEndAction boundaryAction);

        void ComputeDivergence([NotNull] IParticleContext particleContext, [NotNull] GetFluidVector getFluidVector, [NotNull] GetBoundaryVector getBoundaryVector, [CanBeNull] FluidDivergenceEndAction fluidAction, [CanBeNull] BoundaryDivergenceEndAction boundaryAction);

        #endregion
    }
}
