﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System.Collections.Generic;
using JetBrains.Annotations;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using TamiFlu.Core.Simulation.Kernel;
using TamiFlu.Interfaces.Simulation;
using TamiFlu.Interfaces.Simulation.Kernel;

namespace TamiFlu.UnitTest
{
    [TestClass]
    public sealed class KernelUnitTest
    {
        #region Methods

        [NotNull, ItemNotNull]
        private static IEnumerable<IKernel> FindKernelImplementations()
        {
            //var interfaces = AssemblyLoader.FindInterfaces(typeof(IKernel));
            var kernels = new List<IKernel>();
            //kernels.AddRange(interfaces.Select(kvp => (IKernel) Activator.CreateInstance(kvp.Value)));
            kernels.Add(new CubicSplineKernel());
            return kernels;
        }

        [TestMethod]
        public void TestKernelValue()
        {
            GlobalConstants.Instance.Spacing = 1.0f;

            foreach (var kernel in FindKernelImplementations())
            {
                var sum = 0.0f;
                foreach (var x2 in KernelExtensions.SampleVolume(kernel, out var x1, out var sampleVolume))
                {
                    sum += sampleVolume * kernel.ComputeValue(in x1, in x2);
                }

                Assert.AreEqual(1.0f, sum, 1e-3f);
            }
        }

        [TestMethod]
        public void TestKernelGradient()
        {
            GlobalConstants.Instance.Spacing = 1.0f;

            foreach (var kernel in FindKernelImplementations())
            {
                var sum = Vector.Zero;
                foreach (var x2 in KernelExtensions.SampleVolume(kernel, out var x1, out var sampleVolume))
                {
                    sum += sampleVolume * kernel.ComputeGradient(in x1, in x2);
                }

                for (var dimension = 0; dimension < Vector.Dimensions; ++dimension)
                {
                    Assert.AreEqual(0.0f, sum[dimension], 1e-3f);
                }
            }
        }

        #endregion
    }
}
