﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Globalization;
using System.IO;
using System.Threading;
using Eto.Forms;
using JetBrains.Annotations;
using TamiFlu.Core.UI;
using TamiFlu.Interfaces;

namespace TamiFlu.Windows
{
    internal static class Program
    {
        #region Methods

        private static void SetIcon([NotNull] Window window)
        {
            using (var icon = Icon.ExtractAssociatedIcon(typeof(Program).Assembly.Location))
            {
                if (!(icon is null))
                {
                    using (var memoryStream = new MemoryStream())
                    {
                        using (var bitmap = icon.ToBitmap())
                        {
                            bitmap.Save(memoryStream, ImageFormat.Png);
                        }

                        using (var etoBitmap = new Eto.Drawing.Bitmap(memoryStream))
                        {
                            window.Icon = new Eto.Drawing.Icon(1f, etoBitmap);
                        }
                    }
                }
            }
        }

        #endregion

        #region Entry Point

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        public static void Main([NotNull, ItemNotNull] string[] args)
        {
            AssemblyLoader.IgnoreAssemblies("Eto", "JetBrains", "MSFT", "System");

            Thread.CurrentThread.CurrentCulture = CultureInfo.InvariantCulture;
            Thread.CurrentThread.CurrentUICulture = CultureInfo.InvariantCulture;

            var options = ProgramOptions.Parse(args);

            switch (options.Platform.ToLowerInvariant())
            {
                case "direct2d":
                    Eto.Platform.Initialize(Eto.Platforms.Direct2D);
                    break;

                case "winforms":
                    Eto.Platform.Initialize(Eto.Platforms.WinForms);
                    break;

                case "wpf":
                    Eto.Platform.Initialize(Eto.Platforms.Wpf);
                    break;

                case "gtk":
                    Eto.Platform.Initialize(Eto.Platforms.Gtk);
                    break;

                default:
                    throw new ArgumentOutOfRangeException(nameof(options.Platform), options.Platform, null);
            }

            var platform = Eto.Platform.Detect;

            if (platform.IsWinForms)
            {
                platform.Add<PropertyEditorControl.IPropertyEditorHandler>(() => new WinFormsPropertyEditorHandler());
            }

            using (var application = new Application(platform))
            {
                using (var mainForm = new MainForm(args))
                {
                    SetIcon(mainForm);

                    application.Run(mainForm);
                }
            }
        }

        #endregion
    }
}
