﻿#region Copyright
// Copyright (c) 2017 Stefan Band <bands@informatik.uni-freiburg.de>
//  
// Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, 
// provided that the above copyright notice and this permission notice appear in all copies.
// 
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
// INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE
// FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, 
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#endregion
using System.Globalization;
using CommandLine;
using JetBrains.Annotations;

namespace TamiFlu.Windows
{
    internal sealed class ProgramOptions
    {
        #region Constants

        private const string DefaultPlatform = "Direct2D";

        #endregion

        #region Properties

        [Option('p', "platform", Default = DefaultPlatform)]
        public string Platform { get; set; } = DefaultPlatform;

        #endregion

        #region Methods

        [NotNull]
        public static ProgramOptions Parse([NotNull, ItemNotNull] params string[] args)
        {
            ParserResult<ProgramOptions> parserResult;
            using (var parser = new Parser(with => with.ParsingCulture = CultureInfo.InvariantCulture))
            {
                parserResult = parser.ParseArguments<ProgramOptions>(args);
            }

            return parserResult.MapResult(options => options, error => new ProgramOptions());
        }

        #endregion
    }
}
